/*
 * AntiAliasPlugin.java
 * Copyright (C) 2001 Dirk Moebius
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Vector;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.EBMessage;
import org.gjt.sp.jedit.EBPlugin;
import org.gjt.sp.jedit.EditPane;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.gui.OptionsDialog;
import org.gjt.sp.jedit.msg.EditPaneUpdate;
import org.gjt.sp.jedit.msg.PropertiesChanged;
import org.gjt.sp.jedit.textarea.JEditTextArea;
import org.gjt.sp.jedit.textarea.TextAreaHighlight;
import org.gjt.sp.util.Log;


public class AntiAliasPlugin extends EBPlugin
{

	public void start() {
		jEdit.addAction(new ToggleAntiAliasAction());
	}


	public void createMenuItems(Vector menuItems) {
		menuItems.addElement(GUIUtilities.loadMenuItem("toggle-antialias"));
	}


	public void createOptionPanes(OptionsDialog optionsDialog) {
		optionsDialog.addOptionPane(new AntiAliasOptionPane());
	}


	public void handleMessage(EBMessage message) {
		if (message instanceof EditPaneUpdate) {
			EditPaneUpdate eu = (EditPaneUpdate) message;
			EditPane editPane = eu.getEditPane();
			JEditTextArea textArea = editPane.getTextArea();
			Object what = eu.getWhat();
			if (what == EditPaneUpdate.CREATED) {
				AntiAliasHighlight highlight = new AntiAliasHighlight();
				textArea.getPainter().addCustomHighlight(highlight);
				highlighters.put(textArea, highlight);
			}
			else if (what == EditPaneUpdate.DESTROYED) {
				highlighters.remove(textArea);
			}
		}
		else if (message instanceof PropertiesChanged) {
			for(Enumeration e = highlighters.elements(); e.hasMoreElements(); )
				((AntiAliasHighlight)e.nextElement()).propertiesChanged();
		}
	}


	public static boolean isEnabled(JEditTextArea textArea) {
		AntiAliasHighlight highlight = (AntiAliasHighlight) highlighters.get(textArea);
		return highlight != null && highlight.isEnabled();
	}


	public static void toggleEnabled(JEditTextArea textArea) {
		AntiAliasHighlight highlight = (AntiAliasHighlight) highlighters.get(textArea);
		if (highlight != null)
			highlight.toggleEnabled();
	}


	private static Hashtable highlighters = new Hashtable();


	static class AntiAliasHighlight implements TextAreaHighlight
	{
		public AntiAliasHighlight() {
			propertiesChanged();
		}


		public void init(JEditTextArea textArea, TextAreaHighlight next) {
			this.textArea = textArea;
			this.next = next;
		}


		public void paintHighlight(Graphics gfx, int line, int y) {
			HashMap renderHints = new HashMap();

			if (isEnabled) {
				renderHints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				renderHints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
				renderHints.put(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
				renderHints.put(RenderingHints.KEY_FRACTIONALMETRICS,
					useFractionalMetrics ?
						RenderingHints.VALUE_FRACTIONALMETRICS_ON
						: RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
			} else {
				renderHints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
			}

			((Graphics2D)gfx).setRenderingHints(new RenderingHints(renderHints));

			if (next != null)
				next.paintHighlight(gfx, line, y);
		}


		public String getToolTipText(MouseEvent evt) {
			return (next != null ? next.getToolTipText(evt) : null);
		}


		public boolean isEnabled() {
			return isEnabled;
		}


		public void toggleEnabled() {
			isEnabled = !isEnabled;
			if (textArea != null)
				textArea.repaint();
		}


		public void propertiesChanged() {
			isEnabled = jEdit.getBooleanProperty("AntiAliasPlugin.enabled");
			useFractionalMetrics = jEdit.getBooleanProperty("AntiAliasPlugin.fractionalMetrics");
			if (textArea != null)
				textArea.repaint();
		}

		private JEditTextArea textArea = null;
		private TextAreaHighlight next = null;
		private boolean isEnabled = false;
		private boolean useFractionalMetrics = false;
	}

}
